from .serializer import BaseSerializer, serializer
from ..models import jason
from ..models.secret import hash_secrets


@serializer
class JsonCryptSerializer(BaseSerializer):
    """Help text jsoncrypt.
    """
    version = 1
    tag = 'json-crypt'

    def to_json_object(self, secrets):
        res = {
            "version": self.version,
            "from": self.sender,
            "to": self.receiver,
            "secrets": [s.encrypt_to_dict(
                self.sender_private_key,
                self.receiver_public_key
            ) for s in secrets],
            "signature": hash_secrets(secrets),
        }
        return res

    def dumps(self, secrets, system):
        return jason.dumps(self.to_json_object(secrets), indent=4)

    def loads(self, text):
        return text




from seeqret.seeqrypt.nacl_backend import (
    generate_private_key,
    asymetric_encrypt_string, asymetric_decrypt_string,
)


def test_asymetric_roundtrip():
    message = 'hello world'

    A = generate_private_key()
    A_public = A.public_key
    # print()
    # print('A_private:', encoding.Base64Encoder.encode(A._private_key))
    # print('A_public:', encoding.Base64Encoder.encode(bytes(A_public)))
    B = generate_private_key()
    B_public = B.public_key
    # print('B_public:', encoding.Base64Encoder.encode(bytes(B_public)))

    ciphertext = asymetric_encrypt_string(message, A, B_public)
    assert message == asymetric_decrypt_string(ciphertext, B, A_public)



import pytest
import os
import seeqret
from seeqret.run_utils import seeqret_dir
from seeqret.models import Secret
from seeqret.seeqret_init import init_db, create_user_keys
from seeqret.fileutils import remove_file_if_exists
from seeqret.storage.sqlite_storage import SqliteStorage


# create a seeqret db and fill it with 500 secrets
def _create_db():
    create_user_keys('.', 'test', None)
    init_db('.', 'test', 'test@example.com')


def _secret_i(i):
    return Secret(
        app='*',
        env='*',
        key=f'key{i}',
        plaintext_value=str(i),
        type='str'
    )

def _fill_db():
    with seeqret_dir():
        storage = SqliteStorage()
        for i in range(200):
            storage.add_secret(_secret_i(i))


# @pytest.mark.skip(reason="don't test performance in CI")
def test_performance():
    os.environ['TESTING'] = "TRUE"
    _create_db()
    os.environ['SEEQRET'] = '.'
    _fill_db()
    import time
    start = time.time()
    for i in range(2750):
        seeqret.get('key42')
    duration = time.time() - start
    print('Time taken:', duration)
    assert duration < 3
    remove_file_if_exists('private.key')
    remove_file_if_exists('public.key')
    remove_file_if_exists('seeqret.key')
    remove_file_if_exists('seeqrets.db')



# import os

import click

from .console_utils import as_table
from .run_utils import seeqret_dir
from .filterspec import FilterSpec
from .storage.sqlite_storage import SqliteStorage
import logging

logger = logging.getLogger(__name__)


@click.command()
@click.pass_context
@click.argument('filter')
def key(ctx, filter):
    """Remove a secret from the vault.
    """
    with seeqret_dir():
        storage = SqliteStorage()
        spec = FilterSpec(filter)
        logger.debug('remove_secrets: %s', filter)
        if not filter:
            click.secho("ERROR: can't remove all secrets", fg='red')
        secrets = storage.fetch_secrets(**spec.to_filterdict())
        as_table('app,env,key,value,type', secrets)
        if click.confirm('Delete secrets?'):
            print("DELETING SECRETS", [s.key for s in secrets])
        else:
            print("Aborting delete.")

        storage.remove_secrets(**spec.to_filterdict())

        click.secho("secrets deleted.", fg='green')



import os

from nacl.public import PrivateKey
from seeqret.seeqrypt.nacl_backend import (
    generate_private_key,
    load_public_key,
    load_private_key,
    save_private_key,
    save_public_key,
    public_key,
    private_key,
    asymetric_decrypt_string,
    asymetric_encrypt_string,
    sign_message,
    hash_message,
)
from seeqret.fileutils import remove_file_if_exists


def test_generate_private_key():
    # Test successful private key generation
    pkey = generate_private_key('test_generate_private_key.key')
    assert pkey is not None
    assert isinstance(pkey, PrivateKey)
    assert os.path.exists('test_generate_private_key.key')
    remove_file_if_exists('test_generate_private_key.key')


def test_load_private_key():
    # Test loading a valid private key
    valid_private_key = generate_private_key()
    save_private_key('test_load_private_key.key', valid_private_key)
    pubkey = save_public_key('test_load_public_key.key', valid_private_key)
    pubkey2 = public_key(pubkey.decode('ascii'))

    priv = load_private_key('test_load_private_key.key')
    pub = load_public_key('test_load_public_key.key')

    msg = 'hello world'
    cipher = asymetric_encrypt_string(msg, priv, pub)
    plain = asymetric_decrypt_string(cipher, priv, pub)

    assert plain == msg

    remove_file_if_exists('test_load_private_key.key')
    remove_file_if_exists('test_load_public_key.key')


def test_sign_message():
    # Test signing a message
    test_private_key = generate_private_key()
    message = b"Message to sign"
    vkb, signed = sign_message(message)
    assert signed.endswith(b"Message to sign")
    # assert signed.startswith(vkb)


def test_hash_message():
    # Test hashing functionality
    message = b"Message to hash"
    h1 = hash_message(message)
    h2 = hash_message(message + b'.')
    assert h1 is not None
    assert h1 != h2



# from click.testing import CliRunner
#
# from seeqret.db_utils import debug_fetch_users, debug_secrets
# from seeqret.main import cli, user, users, init, list, key
# from tests.clirunner_utils import print_result
# from seeqret.seeqret_transfer import hash_secrets_message, verify_hash
#
#
# def test_hash():
#     runner = CliRunner(env=dict(TESTING="TRUE"))
#     msg = {
#         "data": [
#             {
#                 "app": "yerbu",
#                 "env": "dev",
#                 "key": "LANGCHAIN_API_KEY",
#                 "val": "12345"
#             }
#         ],
#         "from": {
#             "username": "bp",
#             "email": "bp@norsktest.no",
#             "pubkey": "ThkU/1VYhO723bGIYDrPUjczkgNEIsgQLa2R83oYNmQ="
#         },
#         "to": {
#             "username": "bp"
#         }
#     }
#     h = hash_secrets_message(msg)
#     print(h)
#     assert h == 'ugIEK38KzyfbYO3vsoNVCnJdofBel5ttPOnhoh8VCJ8='
#     assert verify_hash(h, msg)
